# Copyright (C) 2023 qBraid
#
# This file is part of the qBraid-SDK
#
# The qBraid-SDK is free software released under the GNU General Public License v3
# or later. You can redistribute and/or modify it under the terms of the GPL v3.
# See the LICENSE file in the project root or <https://www.gnu.org/licenses/gpl-3.0.html>.
#
# THERE IS NO WARRANTY for the qBraid-SDK, as per Section 15 of the GPL v3.

"""
Module providing unified conversions interface between supported
quantum program types.

.. currentmodule:: qbraid.interface

Classes
--------

.. autosummary::
   :toctree: ../stubs/

   ConversionGraph
   ConversionEdge

Functions
-----------

.. autosummary::
   :toctree: ../stubs/

   convert_to_package
   get_qasm_version
   get_program_type
   random_circuit
   random_unitary_matrix

Exceptions
-----------

.. autosummary::
   :toctree: ../stubs/

   NodeNotFoundError
   ConversionPathNotFoundError

"""
from .conversion_edge import ConversionEdge
from .conversion_graph import ConversionGraph
from .converter import convert_to_package
from .exceptions import ConversionPathNotFoundError, NodeNotFoundError
from .inspector import get_program_type, get_qasm_version
from .random import random_circuit, random_unitary_matrix
